from io import BytesIO
import random

import chess
import chess.svg
from cairosvg import svg2png


def generate_board_image(board: chess.Board) -> bytes:
    svg_image = chess.svg.board(board)
    png_image = BytesIO()
    svg2png(bytestring=svg_image, write_to=png_image)
    png_image.seek(0)
    return png_image.read()


def make_move(board: chess.Board, move: str) -> None:
    uci_move = chess.Move.from_uci(move)
    if not board.is_legal(uci_move):
        raise chess.IllegalMoveError
    board.push(uci_move)


def make_bot_move(board: chess.Board) -> None:
    bot_move = random.choice(list(board.legal_moves))
    board.push(bot_move)
