import asyncio
import logging
import sys

from aiogram import Bot, Dispatcher
from aiogram.client.default import DefaultBotProperties
from aiogram.enums import ParseMode
from aiogram.filters import CommandStart, Command
from aiogram.types import Message, BufferedInputFile

import chess

from my_credentials import API_TOKEN
from chess_utils import generate_board_image, make_move, make_bot_move


dp = Dispatcher()
boards = {}


@dp.message(CommandStart())
async def send_welcome(message: Message) -> None:
    await message.reply("Привет! Я ваш первый Telegram-бот.")

@dp.message(Command("newgame"))
async def start_game(message: Message) -> None:
    initial_board = chess.Board()
    boards[message.from_user.id] = initial_board
    board_image = generate_board_image(initial_board)
    await message.answer_photo(
        photo=BufferedInputFile(board_image, "game.png"),
        caption="Игра началась! Делай первый ход. Например: e2e4"
    )

@dp.message()
async def process_move(message: Message) -> None:
    if message.from_user.id not in boards:
        await message.answer("Нет начатой партии. Начните новую игру с помощью команды /newgame")
        return
    board = boards[message.from_user.id]

    try:
        make_move(board, message.text)
    except (chess.InvalidMoveError, chess.IllegalMoveError):
        await message.answer(
            "Некорректный ход. Пожалуйста, запишите ход в формате UCI "
            "https://ru.wikipedia.org/wiki/UCI_(%D0%BF%D1%80%D0%BE%D1%82%D0%BE%D0%BA%D0%BE%D0%BB)"
        )
        return

    outcome = board.outcome()
    if outcome is not None:
        board_image = generate_board_image(board)
        await message.answer_photo(
            photo=BufferedInputFile(board_image, "game.png"),
            caption=f"Игра закончена! Результат - {outcome.result()}. Новая игра - /newgame"
        )
        del boards[message.from_user.id]
        return


    make_bot_move(board)
    outcome = board.outcome()
    if outcome is not None:
        board_image = generate_board_image(board)
        await message.answer_photo(
            photo=BufferedInputFile(board_image, "game.png"),
            caption=f"Игра закончена! Результат - {outcome.result()}. Новая игра - /newgame"
        )
        del boards[message.from_user.id]
        return

    board_image = generate_board_image(board)
    await message.answer_photo(
        photo=BufferedInputFile(board_image, "game.png"),
        caption="Текущее состояние доски. Делайте свой следующий ход"
    )


async def main() -> None:
    bot = Bot(token=API_TOKEN, default=DefaultBotProperties(parse_mode=ParseMode.HTML))
    await dp.start_polling(bot)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    asyncio.run(main())
